import wave
from concurrent.futures import ThreadPoolExecutor
from pyaudio import PyAudio
from time import sleep
import threading


__all__ = ["WavFile", "WavPlayer"]


class WavFile:

    def __init__(self, path):
        with wave.open(path, "rb") as f:
            self.width = f.getsampwidth()
            self.channels = f.getnchannels()
            self.rate = f.getframerate()
            self.frames = f.getnframes()
            self.data = f.readframes(self.frames)
            self.duration = float(self.frames) / self.rate

class WavPlayer:
    def __init__(self):
        self.pa = PyAudio()
        self.sounds = dict()
        self.files = dict()
        self.__pool = ThreadPoolExecutor(max_workers=1)

    def add_sound(self, file, name):
        self.sounds[name] = self.pa.open(
            format=self.pa.get_format_from_width(file.width),
            channels=file.channels,
            rate=file.rate,
            output=True)
        self.files[name] = file

    def stop(self):
        for name in self.sounds.keys():
            self.sounds[name].stop_stream()
            self.sounds[name].close()

    def play(self, sound_name, count=1, pause=None):
        def action(sound_name, count, pause):
            for i in range(count):
                self.sounds[sound_name].write(self.files[sound_name].data)
                if i + 1 == count:
                    break
                if pause is not None:
                    sleep(pause)
        self.__pool.submit(action, sound_name, count, pause)

#unused class
class WavThread:

    def __init__(self, file):
        self.file = file
        self.play = False
        self.__thread = threading.Thread(target=self.__thread_proc)
        self.__thread.start()

    def stop(self):
        self.play = None
        self.__thread.join()

    def __thread_proc(self):
        pa = PyAudio()
        s = pa.open(
            format=pa.get_format_from_width(self.file.width),
            channels=self.file.channels,
            rate=self.file.rate,
            output=True)
        try:
            while True:
                if self.play is None:
                    return
                if self.play:
                    s.write(self.file.data)
                else:
                    sleep(0.05)
        finally:
            s.stop_stream()
            s.close()


if __name__ == "__main__":
    f = WavFile("C:\\Windows\\Media\\tada.wav")
    p = WavPlayer(3)
    p.play(f, count=5, pause=2)
